<?php
namespace Teknisa\CRM\Controller;

use Teknisa\CRM\Controller\TeknisaCrud;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Model\Util\NovoCodigo;
use Zeedhi\Framework\DTO;

class OrganizacaoOperadorController extends TeknisaCrud {
    protected $dataSourceName = 'crm_organOper';
    protected $parameterBagColumns = array("P_CDOPERADOR");
    protected $dataSourceManager;
    
    public function __construct($crudService, ParameterBag $parameterBag, Manager $dataSourceManager, $entityManager) {
        $this->dataSourceManager = $dataSourceManager;
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function findExcetoOper(DTO\Request\Filter $request, DTO\Response $response){
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName('crm_organizacaoExcetoOper');
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $this->dataSourceName = 'crm_organOper';
        parent::find($request, $response);
    }
    
   
    public function setOrganOper(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            foreach ($request->getDataSet()->getRows() as &$row) {
                foreach ($row['NRORGREL'] as $org) {
                    $params = array
                    (
                        "NRORGANIZACAOOPER" => NovoCodigo::geraCodigo($row['NRORG'], "ORGANIZACAOOPER", 1, 4, false, false),
                        "NRORGINCLUSAO"         => $row["NRORG"],
                        "CDOPERADOR"        => $row["CDOPERADOR"],
                        "CDOPERINCLUSAO"    => $row["CDOPERINCLUSAO"],
                        "NRORG"             => $org,
                        "IDATIVO"           => 'S'
                    );
                    $this->insertOrganOper($params);
                }
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
    
    public function insertOrganOper($params)
    {
        $insert = "INSERT INTO ORGANIZACAOOPER
                              (NRORGANIZACAOOPER, NRORGREL, CDOPERADOR, DTINCLUSAO, NRORGINCLUSAO,
                               CDOPERINCLUSAO, NRORG, IDATIVO)
                       VALUES (:NRORGANIZACAOOPER, :NRORG, :CDOPERADOR, SYSDATE, :NRORGINCLUSAO,
                               :CDOPERINCLUSAO, :NRORG, :IDATIVO)";

        try
        {
            $std = $this->entityManager->getConnection()->prepare($insert);
            $std->execute($params);
        }catch(\Exception $e)
        {
            throw new \Exception($e->getMessage());
        };
    }
}
?>