function ABEvolucaoController(CRM_Utilities, templateManager, restEngine, ScreenService, Messages, ZHPromise) {
    
    /**
     * Procedimento para definir a permissão para editar
     * o campo 'Data da Evolução' no ambiente de criação
     * de uma evolução.
     * 
     * @para {widget} widget widget referente a tela de
     * evolução.
    */
    this.setEvolDateChangePermission = function(widget){
        var currentRow = widget.currentRow;
        
        this.getNrDiasLanceEvol(widget).then(function(response){
            var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
            
            if(nrDiasLanceEvol > 0 && currentRow.__is_new){
                widget.getField('DTHREVOLCONT').readOnly = false;
                this.configureEvolDateChangePermission(widget, nrDiasLanceEvol);
            }else{
                widget.getField('DTHREVOLCONT').readOnly = true;
            }
        }.bind(this));
    };
    
    /**
     * Procedimento para buscar, e setar no templateManager
     * (se não já existir no templateManager),
     * o número de dias permitido para uma 
     * evolução ser lançada posteriomento ao dia real.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @return {number} retorna o número de dias pemitido
     * para lançar uma evolução posterior a data real.
    */
    this.getNrDiasLanceEvol = function(widget){
        var nrDiasLanceEvol = templateManager.NRDIASLANCEVOL;
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var deferred = ZHPromise.defer();
        
        if(nrDiasLanceEvol){
            deferred.resolve();
            
            return deferred;
        }else{
            
            var paramsRestEngine = {
            "requestType": "Row",
            "row": {NRORG: loggedOperatorInfos.NRORG}
            };
        
            return restEngine.post('/getNrDiasLanceEvol', paramsRestEngine,
            function(success){
                templateManager.NRDIASLANCEVOL = success.dataset.DADOS.NRDIASLANCEVOL[0].NRDIASLANCEVOL;
            },undefined,
            function(error){
                Messages.CATCH_EXCEPTION_ERROR(error.data.error);
            },undefined,undefined
        );
            
        }
    };
    
    /**
     * Procedimento para configurar as permissões
     * de data máxima e data mínima para lançar
     * uma evolução.
     * 
     * @param {object} widget widget referente a
     * tela de evoluções.
     * 
     * @param {number} numberOfDays número máximo de dias
     * para se lançar uma evolução posteriormente.
    */
    this.configureEvolDateChangePermission = function(widget, numberOfDays){
        var evolDateField = widget.getField('DTHREVOLCONT');
        
        evolDateField.minDate = CRM_Utilities.subtractDateByDays(CRM_Utilities.getCurrentDateWithMinTime(),numberOfDays);
        evolDateField.maxDate = CRM_Utilities.getCurrentDateWithMaxTime();
    };
    
    /**
     * //TODO
    */
    this.configureEvolTimeShowPermission = function(widget){
        var currentRow = widget.currentRow;
        var selectedDateWithoutTime = moment(currentRow.DTHREVOLCONT, 'DD/MM/YYYY HH:mm').format('DD/MM/YYYY');    
        
        if(selectedDateWithoutTime === CRM_Utilities.getCurrentDate()){
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' ' + CRM_Utilities.getCurrentTime();
        }else{
            currentRow.DTHREVOLCONT = selectedDateWithoutTime + ' 00:00';
        }
    };
    
    /**
     * Define o filtro da widget de evolução
     * pelo número do parceiro de negócio selecionado
     * anteriormente.
     * 
     * @param {object} widgetName nome da widget de evolução
     * em caso.
    */
    this.setEvolutionWidgetFilterByNrParcNegocio = function(widgetName){
        
        var evolutionWidget = templateManager.container.getWidget(widgetName);
        
        var widgetFilter = evolutionWidget.dataSourceFilter;
        var parentWidgetCurrentRow = evolutionWidget.getParent().currentRow;
        var nrParcNegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(widgetFilter, "P_NRPARCNEGOCIO");
        
        widgetFilter[nrParcNegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
        evolutionWidget.reload();
        
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ABEvolucaoController', ABEvolucaoController);
});