function MetasController(templateManager, ScreenService, Utilities, restEngine, General, metaDataFactory, CRM_Utilities, ZHPromise, Dashboard_Controller, ZhFilterPreferences, Messages){
    this.setMeta = function(args) {
        var widget = args;
        if (widget.isValid()) {
            widget.currentRow.VRMINMETA = widget.currentRow.VRMINMETA.toString().split(',')[0].replace(/[^0-9]/g, '');
            return General.widgetSave(widget).then(function(response){
                widget.reload().then(function(){
                    this.applyMaskOnMinValue(widget);
                }.bind(this));
            }.bind(this));
        }
    };
    
    this.applyMaskOnMinValue = function(widget){
        var tipoMeta;
        widget.dataSource.data.forEach(function(row){
           tipoMeta = row['CDTIPOMETA']; 
            switch (tipoMeta) {
                case 'VALOR_OPORTUNIDADES' :
                    if(row['VRMINMETA'].toString().indexOf('R$') === -1 && row['VRMINMETA'].toString().length > 0)
                        row['VRMINMETA'] = 'R$'+row['VRMINMETA']+',00';
                    break;
                case 'CONVERSAO_LEADS' :
                    row['VRMINMETA'] = row['VRMINMETA'].toString().split(',')[0].replace(/[^0-9]/g, '');
                    break;
            }
            if(widget.currentRow.NRMETAS === row['NRMETAS'])
                widget.currentRow.VRMINMETA = row['VRMINMETA'];
        });
    };
    
    /**
     * Função para salvar uma meta
     * ao perfil de atividade em caso.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro de meta
    */
    this.salvarMetaNoPerfil = function(widget){
        
        var currentRow = widget.currentRow;
        
        var activityProfileMeta = currentRow.METAS;
        var rows = [];
        
        activityProfileMeta.forEach(function(result, metaIndex){
            
            rows.push(this.setActivityProfileMetaValuesInCurrentRow(widget, metaIndex));
                
        }.bind(this)); 
        
        General.callRestEngine("product#/crm_perfilmetas/save", "DataSet", rows).then(function(response){
                
                rows.forEach(function(meta){
                    var nrmetas = meta['NRMETAS'];
                    var nrseqperfativ = meta['NRSEQPERFATIV'];
                    var cdtipometa = meta['CDTIPOMETA'];
                    this.buildOperadoresPerfil(nrseqperfativ, nrmetas, cdtipometa).then(function(operadores){
                        
                        General.callRestEngine("product#/crm_perfilmetas/saveOperador", "DataSet", operadores).then(function(response){
                    
                                
                        }.bind(this)).catch(function(err) {
                            var errorMessage = Messages.CATCH_EXCEPTION_ERROR(err);
                            ScreenService.showMessage(errorMessage);
                        });  
                        
                    }.bind(this));
                }.bind(this));
                ScreenService.closeSwipe().then(function(){
                    widget.reload();
                });
                
        }.bind(this));
    };
    
    /**
     * Função para pegar as informações do 
     * array de metas na currentRow, e
     * tratá-las em uma cópia da currentRow para
     * realizar o save.
     * 
     * @param {object} widget widget ativa onde está
     * sendo realizado o cadastro de metas.
     * 
     * @param {int} metaIndex index da meta
     * no array de metas presente na currentRow.
    */
    this.setActivityProfileMetaValuesInCurrentRow = function(widget, metaIndex){
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.__is_new = true;
        currentRow.NRSEQPERFATIV = widget.parent.parent.currentRow.NRSEQPERFATIV;
        currentRow.NRMETAS = currentRow.METAS[metaIndex];
        currentRow.CDTIPOMETA = currentRow.CDTIPOMETA[metaIndex];
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        
        return currentRow;
        
    };
    
    /**
     * Função para inserir um filtro na field
     * de metas da tela de perfil de atividade,
     * atualiza a field.
     * 
     * @param {object} widget widget ativa no
     * modo edit, onde está sendo mostrado o campo
     * de metas.
    */
    this.insertMetaFilter = function(widget){
        
        var metaField = widget.getField("METAS");
        var metaFieldFilter = metaField.dataSourceFilter;
        
        var nrseqperfativIndex = CRM_Utilities.getFilterPosition(metaFieldFilter, "P_NRSEQPERFATIV");
        
        metaFieldFilter[nrseqperfativIndex].value = widget.parent.parent.currentRow.NRSEQPERFATIV;
        
        metaField.reload();
    };
    
    this.setOperMetaValuesInCurrentRow = function(widget, operIndex){
        
        var currentRow = angular.copy(widget.currentRow);
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        
        currentRow.__is_new = true;
        currentRow.NRMETAS = widget.parent.parent.currentRow.NRMETAS;
        currentRow.CDOPERADOR = currentRow.OPERADORES[operIndex];
        currentRow.NRORG = loggedOperatorInfos.NRORG;
        currentRow.NRSEQPERFATIV = null;
        
        return currentRow;
        
    };
    
    this.saveNewOper = function(widget){
        
        var currentRow = widget.currentRow;
        
        var metaOper = currentRow.OPERADORES;
        var rows = [];
        
        metaOper.forEach(function(result, operIndex){
            
            rows.push(this.setOperMetaValuesInCurrentRow(widget, operIndex));
                
        }.bind(this)); 
        
        General.callRestEngine("product#/crm_perfilmetas/saveOperador", "DataSet", rows).then(function(response){
                
                ScreenService.closeSwipe().then(function(){
                    
                    widget.reload();
                    
                });
                
        }.bind(this)).catch(function(err) {
            var errorMessage = Messages.CATCH_EXCEPTION_ERROR(err);
            ScreenService.showMessage(errorMessage);
        });    
        
    };
    
    this.buildOperadoresPerfil = function(nrseqperfativ, nrmetas, cdtipometa){
        
        var loggedOperatorInfos = CRM_Utilities.getLoggedOperatorInfos();
        var promise = ZHPromise.defer();
        
        restEngine.post("product#/crud/crm_perfioper_operadormeta", {'requestType':'FilterData', filter: [{operator: "=", name: "NRSEQPERFATIV", value: nrseqperfativ}, {operator: "=", name: "CDTIPOMETA", value: cdtipometa}]}).
        then(function(response){
                
            var operadores = response.dataset.crm_perfioper_operadormeta;   
            operadores.forEach(function(operador, index, operadores){
                var row  = {};
                
                row.__is_new = true;
                row.NRMETAS = nrmetas;
                row.CDOPERADOR = operador['CDOPERADOR'];
                row.NRORG = loggedOperatorInfos.NRORG;
                row.NRSEQPERFATIV = nrseqperfativ;
                operadores[index] = row;
            
                
            }.bind(this));
            
            promise.resolve(operadores);
                
        }.bind(this));   
        return promise.promise;
    };
    
    this.deleteOperadores = function(nrmetas, nrseqperfativ){
        restEngine.post("product#/crm_perfioper_operador/deleteOperadores", {'requestType':'FilterData', filter: [{operator: "=", name: "NRSEQPERFATIV", value: nrseqperfativ}, {operator: "=", name: "NRMETAS", value: nrmetas}]});
    };
    
    this.deleteMeta = function(widget, nrmetas, nrseqperfativ){
        General.deleteCrud(widget).then(function(){
            this.deleteOperadores(nrmetas, nrseqperfativ)
        }.bind(this));
        
    }
    
    this.changeMonth = function(widget, goTo, betweenFilter){
        var indicadorWdgt = Dashboard_Controller.chooseWidgetChart(widget.chart);
        var mesatual = indicadorWdgt.currentRow.ORDEM;
        var mesMin = moment().subtract(6, 'months').format('MM/YY');
        var mesMax = moment().format('MM/YY');
        
        if((this.compareMonths(mesatual,mesMin) && goTo === 'back') || (this.compareMonths(mesMax,mesatual) && goTo === 'next') || (betweenFilter && goTo === 'back') ){
            var mes = parseInt(mesatual.slice(0,2))+(goTo === 'back' ? -1 : +1);
            var ano = mesatual.slice(3,5);
            
            //Trata anos diferentes
            if(mes == 13){
                mes = 1;
                ano = parseInt(ano) + 1;
            }if(mes == 0){
                mes = 12;
                ano = parseInt(ano) - 1;
            }
            
            var pad = "00";
            mes = (pad+mes).slice(-pad.length);
            if(betweenFilter){
                var filters = [];
                indicadorWdgt.dataSourceFilter.forEach(function(filter, index){
                    if(filter.name !== 'DATAINICIAL' && filter.name !== 'DATAFINAL'){
                        filters.push(filter);
                    }
                });
                filters.push({
                    name:'DATAINICIAL',
                    operator:'=',
                    static:true,
                    value:'01/'+mes+'/'+ano+' 00:00'
                });
                var lastday = new Date(2000+ano, mes, 0).getDate();
                filters.push({
                    name:'DATAFINAL',
                    operator:'=',
                    static:true,
                    value:lastday+'/'+mes+'/'+ano+' 23:59'
                });
                indicadorWdgt.dataSourceFilter = filters;
            }else{
                indicadorWdgt.dataSourceFilter[2] = {
                        name: "ORDEM",
                        operator: "=",
                        static: true,
                        value: mes
                };
            }
            indicadorWdgt.currentRow['ORDEM'] = mes + '/' + ano;
            var stripe = templateManager.container.getWidget('WDGT_INDICADOR_STRIPE');
            stripe.fields[0].label = indicadorWdgt.label;
            stripe.fields[1].label = new Date(mes + '/01/' + ano).toLocaleString('pt-BR', {month:'long', year:'numeric'});
            stripe.fields[1].label = CRM_Utilities.capitalizeFirstLetter(stripe.fields[1].label);
            indicadorWdgt.reload();
        }else if(mesatual === mesMax && goTo === 'next'){
            ScreenService.alertNotification('Não é possível mostrar datas superiores ao mês atual.', 2000);
        }else if(mesatual === mesMin && goTo === 'back'){
            ScreenService.alertNotification('Só é possivel mostrar até 6 meses anteriores ao mês atual.', 2000);
        }
        
        
    };
    
    
    //Retorna true se o primeiro mês é menor que o segundo, verifica ano
    this.compareMonths = function(month1, month2){
        var year1 = month1.slice(3,5);
        var year2 = month2.slice(3,5);
        
        if(year1 > year2){
            return true;
        }else if(month1 > month2){
            return true;
        }
        
        return false;
        
    };
    
    this.openOportWindowFilteringOper = function(widget){
        ScreenService.openWindow('product#crm00002_001_oportunidade_de_negocio').then(function(){
            
            var oportWidgt = templateManager.container.getWidget('oportunidadegrid');
            var date = new Date();
            var firstDateBewteen = new Date(date.getFullYear(), (parseInt(widget.currentRow.ORDEM)-1), 1);
            var lastDateBewteen  = new Date(date.getFullYear(), (parseInt(widget.currentRow.ORDEM)), 0);
            firstDateBewteen = firstDateBewteen.toLocaleDateString('pt-BR');
            lastDateBewteen = lastDateBewteen.toLocaleDateString('pt-BR');
            
            oportWidgt.dataSourceFilter = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: "@templateManager.LOGGEDOPERATOR.NRORG@"
                },
                {
                    name: "IDTPOPERADOR",
                    operator: "=",
                    value: "E"
                },
                {
                    name: "NRPARCRESP",
                    operator: "=",
                    default: true,
                    value: widget.currentRow.NRPARCNEGOCIO
                },
                {
                    name: "SITUACAO",
                    operator: "IN",
                    static: true,
                    value: "@['F']@"
                },
                {
                    name: "DINCLUSAO",
                    operator: "BETWEEN",
                    static: true,
                    value: [firstDateBewteen,lastDateBewteen]
                }
            ];
            oportWidgt.reload().then(function(){
                ScreenService.closePopup(true);
                oportWidgt.actions.forEach(function(action, index){
                    if(action.name == 'BACK_CHARTS')
                        action.isVisible = true;
                });
                oportWidgt.activate();
                oportWidgt.METAFILTER = true;
                oportWidgt.__filterPopup.currentRow.CDOPERADOR = [widget.currentRow.CDOPERADOR];
                oportWidgt.__filterPopup.currentRow.NRPARCRESP = [widget.currentRow.NRPARCNEGOCIO];
                oportWidgt.__filterPopup.currentRow.NMPARCRESP = [widget.currentRow.OPERADOR];
                oportWidgt.__filterPopup.currentRow.DINCLUSAO = {start: firstDateBewteen, period: undefined, end:lastDateBewteen};
                oportWidgt.__filterPopup.getField('NRPARCRESP').reload();
                oportWidgt.__filterPopup.getField('DINCLUSAO').reload();
            });
            
            
            
        });
    };
    
    this.openProspectWindowFilteringOper = function(widget){
        ScreenService.openWindow('product#ger45000_prospect').then(function(){
            
            var prospWidgt = templateManager.container.getWidget('gridProspect');
            var date = new Date();
            var firstDateBewteen = new Date(date.getFullYear(), (parseInt(widget.currentRow.ORDEM)-1), 1);
            var lastDateBewteen  = new Date(date.getFullYear(), (parseInt(widget.currentRow.ORDEM)), 0);
            firstDateBewteen = firstDateBewteen.toLocaleDateString('pt-BR') ;
            lastDateBewteen = lastDateBewteen.toLocaleDateString('pt-BR') + ' 23:59';
            
            prospWidgt.dataSourceFilter = [
                {
                    name: "NRORG",
                    operator: "=",
                    value: "@templateManager.LOGGEDOPERATOR.NRORG@"
                },
                {
                    name: "IDTPOPERADOR",
                    operator: "=",
                    value: "E"
                },
                {
                    name: "CDOPERINCLUSAO",
                    operator: "=",
                    static:true,
                    value: widget.currentRow.CDOPERADOR
                },
                {
                    name: "DATAINCLUSAO",
                    operator: "BETWEEN",
                    static:true,
                    value: [firstDateBewteen,lastDateBewteen]
                }
            ];
            prospWidgt.reload().then(function(){
                ScreenService.closePopup(true);
                prospWidgt.actions.forEach(function(action, index){
                    if(action.name == 'BACK_CHARTS')
                        action.isVisible = true;
                });
                prospWidgt.activate();
                prospWidgt.METAFILTER = true;
                prospWidgt.__filterPopup.currentRow.NRPARCRESPROSP = [widget.currentRow.NRPARCNEGOCIO];
                prospWidgt.__filterPopup.currentRow.NMPARCRESPROSP = [widget.currentRow.OPERADOR];
                prospWidgt.__filterPopup.currentRow.DATAINCLUSAO = {start: firstDateBewteen, period: undefined, end:lastDateBewteen};
                prospWidgt.__filterPopup.getField('NMPARCRESPROSP').reload();
                prospWidgt.__filterPopup.getField('DATAINCLUSAO').reload();
            });
            
            
            
        });
    };
    
    this.goBackCharts = function(){
        ScreenService.goBack();
        var oportWidgt = templateManager.container.getWidget('oportunidadegrid');
        oportWidgt.actions.forEach(function(action, index){
            if(action.name == 'BACK_CHARTS')
                action.isVisible = false;
        });
        oportWidgt.activate();
    };
    
    this.checkMetaFilter = function(widget){
        if(widget.METAFILTER){
            ZhFilterPreferences.openFilterWidget(widget);
            widget.METAFILTER = false;
            widget.actions.forEach(function(action, index){
            if(action.name == 'BACK_CHARTS')
                action.isVisible = false;
            });
        }
    };
    
    this.setFiltroPai = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.dataSourceFilter.find(x => x.name == 'NRSEQPERFATIV').value = widget.parent.parent.currentRow.NRSEQPERFATIV;
    };
	
}
Configuration(function(ContextRegister){
    ContextRegister.register('MetasController', MetasController);
});